@@ -83,6 +83,18 @@ class LatticeU1(BaseLatticeU1):
         local_action = tf.ones_like(wloops) - tf.math.cos(wloops)
         return beta * tf.reduce_sum(local_action, (1, 2))
 
+    def grad_action(self, x: Tensor, beta: Tensor) -> Tensor:
+        """Compute the gradient of the potential function."""
+        if tf.executing_eagerly():
+            with tf.GradientTape() as tape:
+                tape.watch(x)
+                pe = self.action(x, beta)
+            grad = tape.gradient(pe, x)
+        else:
+            grad = tf.gradients(self.action(x, beta), [x])[0]
+
+        return grad
+
     def calc_metrics(
             self,
             x: Tensor,
